@@ -313,6 +313,13 @@ class MultiModalClassificationModel:
         global_step, tr_loss = self.train(
             train_dataset,
             output_dir,
+            files_list=files_list,
+            image_path=image_path,
+            text_label=text_label,
+            labels_label=labels_label,
+            images_label=images_label,
+            image_type_extension=image_type_extension,
+            data_type_extension=data_type_extension,
             show_running_loss=show_running_loss,
             eval_data=eval_data,
             verbose=verbose,
@@ -325,7 +332,20 @@ class MultiModalClassificationModel:
             logger.info(" Training of {} model complete. Saved to {}.".format(self.args.model_type, output_dir))
 
     def train(
-        self, train_dataset, output_dir, show_running_loss=True, eval_data=None, verbose=True, **kwargs,
+        self,
+        train_dataset,
+        output_dir,
+        files_list=None,
+        image_path=None,
+        text_label=None,
+        labels_label=None,
+        images_label=None,
+        image_type_extension=None,
+        data_type_extension=None,
+        show_running_loss=True,
+        eval_data=None,
+        verbose=True,
+        **kwargs,
     ):
         """
         Trains the model on train_dataset.
@@ -351,16 +371,58 @@ class MultiModalClassificationModel:
         t_total = len(train_dataloader) // args.gradient_accumulation_steps * args.num_train_epochs
 
         no_decay = ["bias", "LayerNorm.weight"]
-        optimizer_grouped_parameters = [
-            {
-                "params": [p for n, p in model.named_parameters() if not any(nd in n for nd in no_decay)],
-                "weight_decay": args.weight_decay,
-            },
-            {
-                "params": [p for n, p in model.named_parameters() if any(nd in n for nd in no_decay)],
-                "weight_decay": 0.0,
-            },
-        ]
+
+        optimizer_grouped_parameters = []
+        custom_parameter_names = set()
+        for group in self.args.custom_parameter_groups:
+            params = group.pop("params")
+            custom_parameter_names.update(params)
+            param_group = {**group}
+            param_group["params"] = [p for n, p in model.named_parameters() if n in params]
+            optimizer_grouped_parameters.append(param_group)
+
+        for group in self.args.custom_layer_parameters:
+            layer_number = group.pop("layer")
+            layer = f"layer.{layer_number}."
+            group_d = {**group}
+            group_nd = {**group}
+            group_nd["weight_decay"] = 0.0
+            params_d = []
+            params_nd = []
+            for n, p in model.named_parameters():
+                if n not in custom_parameter_names and layer in n:
+                    if any(nd in n for nd in no_decay):
+                        params_nd.append(p)
+                    else:
+                        params_d.append(p)
+                    custom_parameter_names.add(n)
+            group_d["params"] = params_d
+            group_nd["params"] = params_nd
+
+            optimizer_grouped_parameters.append(group_d)
+            optimizer_grouped_parameters.append(group_nd)
+
+        if not self.args.train_custom_parameters_only:
+            optimizer_grouped_parameters.extend(
+                [
+                    {
+                        "params": [
+                            p
+                            for n, p in model.named_parameters()
+                            if n not in custom_parameter_names and not any(nd in n for nd in no_decay)
+                        ],
+                        "weight_decay": args.weight_decay,
+                    },
+                    {
+                        "params": [
+                            p
+                            for n, p in model.named_parameters()
+                            if n not in custom_parameter_names and any(nd in n for nd in no_decay)
+                        ],
+                        "weight_decay": 0.0,
+                    },
+                ]
+            )
 
         warmup_steps = math.ceil(t_total * args.warmup_ratio)
         args.warmup_steps = warmup_steps if args.warmup_steps == 0 else args.warmup_steps
@@ -393,15 +455,20 @@ class MultiModalClassificationModel:
             training_progress_scores = self._create_training_progress_scores(multi_label, **kwargs)
 
         if args.wandb_project:
-            wandb.init(project=args.wandb_project, config={**args}, **args.wandb_kwargs)
+            wandb.init(project=args.wandb_project, config={**asdict(args)}, **args.wandb_kwargs)
             wandb.watch(self.model)
 
         model.train()
         for _ in train_iterator:
             train_iterator.set_description(f"Epoch {epoch_number} of {args.num_train_epochs}")
-            for step, batch in enumerate(
-                tqdm(train_dataloader, desc=f"Running Epoch {epoch_number}", disable=args.silent)
-            ):
+            train_iterator.set_description(f"Epoch {epoch_number + 1} of {args.num_train_epochs}")
+            batch_iterator = tqdm(
+                train_dataloader,
+                desc=f"Running Epoch {epoch_number} of {args.num_train_epochs}",
+                disable=args.silent,
+                mininterval=0,
+            )
+            for step, batch in enumerate(batch_iterator):
                 batch = tuple(t.to(device) for t in batch)
                 labels = batch[5]
 
@@ -417,7 +484,9 @@ class MultiModalClassificationModel:
                 current_loss = loss.item()
 
                 if show_running_loss:
-                    print("\rRunning loss: %f" % loss, end="")
+                    batch_iterator.set_description(
+                        f"Epochs {epoch_number}/{args.num_train_epochs}. Running Loss: {current_loss:9.4f}"
+                    )
 
                 if args.gradient_accumulation_steps > 1:
                     loss = loss / args.gradient_accumulation_steps
@@ -473,6 +542,13 @@ class MultiModalClassificationModel:
                         # Only evaluate when single GPU otherwise metrics may not average well
                         results, _ = self.eval_model(
                             eval_data,
+                            files_list=files_list,
+                            image_path=image_path,
+                            text_label=text_label,
+                            labels_label=labels_label,
+                            images_label=images_label,
+                            image_type_extension=image_type_extension,
+                            data_type_extension=data_type_extension,
                             verbose=verbose and args.evaluate_during_training_verbose,
                             silent=args.evaluate_during_training_silent,
                             **kwargs,
@@ -551,6 +627,13 @@ class MultiModalClassificationModel:
             if args.evaluate_during_training:
                 results, _ = self.eval_model(
                     eval_data,
+                    files_list=files_list,
+                    image_path=image_path,
+                    text_label=text_label,
+                    labels_label=labels_label,
+                    images_label=images_label,
+                    image_type_extension=image_type_extension,
+                    data_type_extension=data_type_extension,
                     verbose=verbose and args.evaluate_during_training_verbose,
                     silent=args.evaluate_during_training_silent,
                     **kwargs,
@@ -889,7 +972,7 @@ class MultiModalClassificationModel:
 
         if self.model.num_labels == 2:
             tn, fp, fn, tp = confusion_matrix(labels, preds).ravel()
-            return ({**{"mcc": mcc, "tp": tp, "tn": tn, "fp": fp, "fn": fn}, **extra_metrics},)
+            return {**{"mcc": mcc, "tp": tp, "tn": tn, "fp": fp, "fn": fn}, **extra_metrics}
         else:
             return {**{"mcc": mcc}, **extra_metrics}
 
@@ -1067,3 +1150,6 @@ class MultiModalClassificationModel:
         args = MultiModalClassificationArgs()
         args.load(input_dir)
         return args
+
+    def get_named_parameters(self):
+        return [n for n, p in self.model.named_parameters()]
