@@ -6,6 +6,7 @@ import random
 import warnings
 from multiprocessing import cpu_count, Pool
 from pathlib import Path
+from dataclasses import asdict
 
 import numpy as np
 from tqdm.auto import tqdm, trange
@@ -40,6 +41,9 @@ from transformers import (
     LongformerConfig,
     LongformerModel,
     LongformerTokenizer,
+    MobileBertConfig,
+    MobileBertModel,
+    MobileBertTokenizer,
     PreTrainedModel,
     PreTrainedTokenizer,
     RobertaConfig,
@@ -70,6 +74,7 @@ MODEL_CLASSES = {
     "distilbert": (DistilBertConfig, DistilBertModel, DistilBertTokenizer),
     "electra": (ElectraConfig, ElectraModel, ElectraTokenizer),
     "longformer": (LongformerConfig, LongformerModel, LongformerTokenizer),
+    "mobilebert": (MobileBertConfig, MobileBertModel, MobileBertTokenizer),
     "marian": (MarianConfig, MarianMTModel, MarianTokenizer),
     "roberta": (RobertaConfig, RobertaModel, RobertaTokenizer),
 }
@@ -171,7 +176,7 @@ class Seq2SeqModel:
             if encoder_decoder_type == "bart":
                 self.encoder_tokenizer = tokenizer_class.from_pretrained(encoder_decoder_name)
             elif encoder_decoder_type == "marian":
-                if "base_marian_model_name" in self.args:
+                if self.args.base_marian_model_name:
                     self.encoder_tokenizer = tokenizer_class.from_pretrained(self.args.base_marian_model_name)
                 else:
                     self.encoder_tokenizer = tokenizer_class.from_pretrained(encoder_decoder_name)
@@ -229,7 +234,7 @@ class Seq2SeqModel:
         Args:
             train_data: Pandas DataFrame containing the 2 columns - `input_text`, `target_text`.
                         - `input_text`: The input text sequence.
-                        - `target_text`: The target sequence         
+                        - `target_text`: The target sequence
             output_dir: The directory where model files will be saved. If not given, self.args.output_dir will be used.
             show_running_loss (optional): Set to False to prevent running loss from being printed to console. Defaults to True.
             args (optional): Optional changes to the args dict of the model. Any changes made will persist for the model.
@@ -312,13 +317,58 @@ class Seq2SeqModel:
             t_total = len(train_dataloader) // args.gradient_accumulation_steps * args.num_train_epochs
 
         no_decay = ["bias", "LayerNorm.weight"]
-        optimizer_grouped_parameters = [
-            {
-                "params": [p for n, p in model.named_parameters() if not any(nd in n for nd in no_decay)],
-                "weight_decay": args.weight_decay,
-            },
-            {"params": [p for n, p in model.named_parameters() if any(nd in n for nd in no_decay)]},
-        ]
+
+        optimizer_grouped_parameters = []
+        custom_parameter_names = set()
+        for group in self.args.custom_parameter_groups:
+            params = group.pop("params")
+            custom_parameter_names.update(params)
+            param_group = {**group}
+            param_group["params"] = [p for n, p in model.named_parameters() if n in params]
+            optimizer_grouped_parameters.append(param_group)
+
+        for group in self.args.custom_layer_parameters:
+            layer_number = group.pop("layer")
+            layer = f"layer.{layer_number}."
+            group_d = {**group}
+            group_nd = {**group}
+            group_nd["weight_decay"] = 0.0
+            params_d = []
+            params_nd = []
+            for n, p in model.named_parameters():
+                if n not in custom_parameter_names and layer in n:
+                    if any(nd in n for nd in no_decay):
+                        params_nd.append(p)
+                    else:
+                        params_d.append(p)
+                    custom_parameter_names.add(n)
+            group_d["params"] = params_d
+            group_nd["params"] = params_nd
+
+            optimizer_grouped_parameters.append(group_d)
+            optimizer_grouped_parameters.append(group_nd)
+
+        if not self.args.train_custom_parameters_only:
+            optimizer_grouped_parameters.extend(
+                [
+                    {
+                        "params": [
+                            p
+                            for n, p in model.named_parameters()
+                            if n not in custom_parameter_names and not any(nd in n for nd in no_decay)
+                        ],
+                        "weight_decay": args.weight_decay,
+                    },
+                    {
+                        "params": [
+                            p
+                            for n, p in model.named_parameters()
+                            if n not in custom_parameter_names and any(nd in n for nd in no_decay)
+                        ],
+                        "weight_decay": 0.0,
+                    },
+                ]
+            )
 
         warmup_steps = math.ceil(t_total * args.warmup_ratio)
         args.warmup_steps = warmup_steps if args.warmup_steps == 0 else args.warmup_steps
@@ -386,7 +436,7 @@ class Seq2SeqModel:
             training_progress_scores = self._create_training_progress_scores(**kwargs)
 
         if args.wandb_project:
-            wandb.init(project=args.wandb_project, config={**args}, **args.wandb_kwargs)
+            wandb.init(project=args.wandb_project, config={**asdict(args)}, **args.wandb_kwargs)
             wandb.watch(self.model)
 
         model.train()
@@ -395,9 +445,13 @@ class Seq2SeqModel:
                 epochs_trained -= 1
                 continue
             train_iterator.set_description(f"Epoch {epoch_number + 1} of {args.num_train_epochs}")
-            for step, batch in enumerate(
-                tqdm(train_dataloader, desc=f"Running Epoch {epoch_number}", disable=args.silent)
-            ):
+            batch_iterator = tqdm(
+                train_dataloader,
+                desc=f"Running Epoch {epoch_number} of {args.num_train_epochs}",
+                disable=args.silent,
+                mininterval=0,
+            )
+            for step, batch in enumerate(batch_iterator):
                 if steps_trained_in_current_epoch > 0:
                     steps_trained_in_current_epoch -= 1
                     continue
@@ -414,7 +468,9 @@ class Seq2SeqModel:
                 current_loss = loss.item()
 
                 if show_running_loss:
-                    print("\rRunning loss: %f" % loss, end="")
+                    batch_iterator.set_description(
+                        f"Epochs {epoch_number}/{args.num_train_epochs}. Running Loss: {current_loss:9.4f}"
+                    )
 
                 if args.gradient_accumulation_steps > 1:
                     loss = loss / args.gradient_accumulation_steps
@@ -629,7 +685,7 @@ class Seq2SeqModel:
         Args:
             eval_data: Pandas DataFrame containing the 2 columns - `input_text`, `target_text`.
                         - `input_text`: The input text sequence.
-                        - `target_text`: The target sequence            
+                        - `target_text`: The target sequence
             output_dir: The directory where model files will be saved. If not given, self.args.output_dir will be used.
             verbose: If verbose, results will be printed to the console on completion of evaluation.
             silent: If silent, tqdm progress bars will be hidden.
@@ -765,9 +821,8 @@ class Seq2SeqModel:
 
             all_outputs.extend(outputs.cpu().numpy())
 
-        self.model.to("cpu")
-
         if self.args.use_multiprocessed_decoding:
+            self.model.to("cpu")
             with Pool(self.args.process_count) as p:
                 outputs = list(
                     tqdm(
@@ -831,7 +886,8 @@ class Seq2SeqModel:
         if not no_cache:
             no_cache = args.no_cache
 
-        os.makedirs(self.args.cache_dir, exist_ok=True)
+        if not no_cache:
+            os.makedirs(self.args.cache_dir, exist_ok=True)
 
         mode = "dev" if evaluate else "train"
 
@@ -946,3 +1002,6 @@ class Seq2SeqModel:
         args = Seq2SeqArgs()
         args.load(input_dir)
         return args
+
+    def get_named_parameters(self):
+        return [n for n, p in self.model.named_parameters()]
